@@ -13,7 +13,9 @@ from interface import implements
 from xtlib import utils
 from xtlib import errors
 from xtlib import scriptor
+from xtlib import constants
 from xtlib import file_utils
+from xtlib import run_helper
 
 from xtlib.console import console
 from xtlib.hparams import hp_helper
@@ -46,10 +48,11 @@ class AzureML(BackendBase):
         self.request = None
 
         # load azure libraries on demand
-        global Workspace, Experiment, Run
+        global Workspace, Experiment, Run, ComputeTarget
         import azureml.core
         from azureml.core import Experiment, Run
         from azureml.core.workspace import Workspace
+        from azureml.core.compute import ComputeTarget
 
         if disable_warnings:
             # turn off all azure warnings
@@ -67,7 +70,7 @@ class AzureML(BackendBase):
             storage_name, storage_key = self.get_action_args(args)
 
         # local or POOL of vm's
-        wrapped_parts = None     # we use same cmd on each box/job
+        fn_wrapped = None     # we use same script for each box (but with different ARGS)
         username = args["username"]
 
         for i, box_runs in enumerate(job_runs):
@@ -77,11 +80,12 @@ class AzureML(BackendBase):
             actions = ["data", "model"]
             run_name = br["run_name"]
             is_windows = False
+            node_id = utils.node_id(i)
 
             run_specs = br["run_specs"]
             cmd_parts = run_specs["cmd_parts"]
 
-            if not wrapped_parts:
+            if not fn_wrapped:
                 # just wrap the user cmd once (shared by all boxes/nodes)
                 assert cmd_parts[0] == "python"
                 assert cmd_parts[1] == "-u"
@@ -92,28 +96,61 @@ class AzureML(BackendBase):
                 arg_parts = cmd_parts[3:]
 
                 setup = self.config.get_setup_from_target_def(self.compute_def)
+                use_sudo = utils.safe_value(setup, "use-sudo")       # some AML clusters need sudo
 
-                # we only do this once (for the first box/job)
-                sh_parts = super().wrap_user_command(cmd_parts, snapshot_dir, store_data_dir, data_action, 
+                fn_wrapped = self.create_wrapper_script(cmd_parts, snapshot_dir, store_data_dir, data_action, 
                     data_writable, store_model_dir, model_action, model_writable, storage_name, storage_key, actions, 
-                    is_windows=is_windows, sudo_available=False, username=username, use_username=False, 
-                    install_blobfuse=True, setup=setup, run_name=run_name, change_dir=False, args=args)
+                    use_sudo, username, setup, use_allow_other=use_sudo, args=args)
 
-                # AML wants a python script, so use our tiny python shim to run wrapped.sh 
-                fn_shim = "aml_shim.py"
-                fn_from = file_utils.get_xtlib_dir() + "/" + fn_shim
-                fn_to = snapshot_dir + "/" + fn_shim
+                # AML wants a python script, so use our tiny python shim to run __wrapped__.sh 
+                bootstrap_dir = args["bootstrap_dir"]
+                fn_shim = constants.FN_AML_SHIM
+                fn_from = file_utils.get_xtlib_dir() + "/backends/" + constants.FN_AML_SHIM
+                fn_to = bootstrap_dir + "/" + fn_shim
                 shutil.copyfile(fn_from, fn_to)
 
                 # copy to submit-logs
                 utils.copy_to_submit_logs(args, fn_from)
 
-                # pass sh_parts as a single argument to avoid wierd "arg": 1 problems with AML estimators
-                wrapped_parts = ["python", "-u", fn_shim, " ".join(sh_parts)]
+            # we update each box's command (passing RUN_NAME as arg to __wrapped__.sh)
+            script_part = "{} {} {}".format(os.path.basename(fn_wrapped), i, run_name)
+            sh_parts = ['/bin/bash', '--login', script_part]
 
-            # we update each box's command
+            # pass sh_parts as a single argument to avoid wierd "arg": 1 problems with AML estimators
+            wrapped_parts = ["python", "-u", fn_shim, " ".join(sh_parts)]
             run_specs["cmd_parts"] = wrapped_parts
 
+    def get_default_docker_image(self, use_gpu):
+        if use_gpu:
+            # recommended for IPT w/GPU: 
+            #mcr.microsoft.com/azureml/base-gpu:openmpi3.1.2-cuda10.0-cudnn7-ubuntu16.04
+            aml_default = "mcr.microsoft.com/azureml/base-gpu:intelmpi2018.3-cuda9.0-cudnn7-ubuntu16.04"
+        else:
+            aml_default = "mcr.microsoft.com/azureml/base:intelmpi2018.3-ubuntu16.04"
+
+        return aml_default
+
+    def create_wrapper_script(self, cmd_parts, snapshot_dir, store_data_dir, data_action, 
+            data_writable, store_model_dir, model_action, model_writable, storage_name, storage_key, actions, 
+             sudo_available, username, setup, use_allow_other, args):
+        '''
+        ITP subclass overwrites this method.
+        '''
+        # we keep the working directory assigned to us by AML because our code is already 
+        # downloaded here, and the log files are handy to access (as subdirs)
+
+        use_gpu = args["use_gpu"]
+        self.default_docker_image = self.get_default_docker_image(use_gpu)
+
+        # we only do this once (for the first box/job)
+        fn_wrapped = super().wrap_user_command(cmd_parts, snapshot_dir, store_data_dir, data_action, 
+            data_writable, store_model_dir, model_action, model_writable, storage_name, storage_key, actions, 
+            is_windows=False, sudo_available=sudo_available, username=username, use_username=False, 
+            install_blobfuse=True, setup=setup,  use_allow_other=use_allow_other, 
+            remove_zip=False, homebase=".", cwd=".", args=args)
+
+        return fn_wrapped
+
     def provides_container_support(self):
         '''
         Returns:
@@ -314,7 +351,7 @@ class AzureML(BackendBase):
             else:
                 run_name = ws_run_name
 
-            if run_name.startswith("run"):
+            if run_helper.is_run_name(run_name):
                 run_name = exper_name + "." + run_name
             result = self.cancel_run(ws_name, run_name)
             results.append(result)
@@ -322,7 +359,7 @@ class AzureML(BackendBase):
         results_by_aml = {"Azure ML": results}
         return results_by_aml
 
-    def get_active_jobs(self):
+    def get_active_jobs(self, ws_name):
         ''' return a list of job_id's running on this instance of Azure Batch '''
         mongo = self.store.get_mongo()
 
@@ -336,11 +373,11 @@ class AzureML(BackendBase):
 
         fields_dict = {"job_id": 1, "service_info_by_node": 1, "service_job_info": 1}
 
-        job_records = mongo.get_info_for_jobs(filter_dict, fields_dict)
+        job_records = mongo.get_info_for_jobs(ws_name, filter_dict, fields_dict)
 
         return job_records
 
-    def cancel_runs_by_user(self, box_name):
+    def cancel_runs_by_user(self, ws_name, box_name):
         '''
         Args:
             box_name: the name of the box the runs ran on (pool service)
@@ -351,7 +388,7 @@ class AzureML(BackendBase):
         cancel_results = []
 
         # get list of active jobs from batch
-        active_jobs = self.get_active_jobs()
+        active_jobs = self.get_active_jobs(ws_name)
         console.diag("after get_active_jobs()")
 
         if active_jobs:
@@ -385,7 +422,7 @@ class AzureML(BackendBase):
 
         # we build service-based box names to have 3 parts
         job_id, service_name, node_index = box_name.split("-")
-        active_jobs = self.get_active_jobs()
+        active_jobs = self.get_active_jobs(workspace)
         cancel_results = []
         if active_jobs:
             for job_record in active_jobs:
@@ -560,6 +597,35 @@ class AzureML(BackendBase):
 
         return trainer
 
+    def get_docker_container_registry(self, args):
+        import azureml
+
+        target = args["target"]
+        docker_name = args["docker"]
+        creds_required = False
+
+        docker_image, login_server, docker_registry = self.config.get_docker_info(target, docker_name, required=False)
+
+        # aml doesn't like/recognized docker.io
+        if login_server == "docker.io":
+            login_server = None
+
+        if not docker_image:
+            docker_image = None
+
+        if login_server:
+            container_registry = azureml.core.ContainerRegistry()
+            container_registry.address = login_server
+            creds_required = utils.make_numeric_if_possible( utils.safe_value(docker_registry, "login") )
+            if creds_required:
+                # get username/password to log into private docker registry service (Azure)
+                container_registry.username = utils.safe_value(docker_registry, "username")
+                container_registry.password = utils.safe_value(docker_registry, "password")
+        else:
+            container_registry = None
+
+        return container_registry, docker_image
+
     def create_estimator(self, job_id, workspace, aml_ws_name, xt_exper_name, aml_exper_name, run_name, code_dir, target_fn, arg_dict, 
         compute_target, node_id, nodes, fake_submit, args):
 
@@ -577,16 +643,24 @@ class AzureML(BackendBase):
             ws = self.get_aml_ws(aml_ws_name)
             experiment = Experiment(ws, name=aml_exper_name)
 
+        # debug code
+        # targets = ComputeTarget.list(ws)
+        # console.print("available compute targets in workspace {}: {}".format(ws, targets))
+
+        # set AML target 
         if compute_target == "amlcompute":
-            actual_target = "amlcompute"    # AmlCompute(ws, None)
+            actual_target = "amlcompute"    
         else:
             if fake_submit:
                 actual_target = "amlcompute"
             else:
-                if not compute_target in ws.compute_targets:
-                    errors.config_error("compute target '{}' does not exist in AML workspace '{}'".format(compute_target, aml_ws_name))
-
-                actual_target = ws.compute_targets[compute_target]
+                if compute_target in ws.compute_targets:
+                    # its the name of a known AML target for this ws (compute instance, cluster, etc.)
+                    actual_target = ws.compute_targets[compute_target]
+                else:
+                    #errors.config_error("compute target '{}' does not exist in AML workspace '{}'".format(compute_target, aml_ws_name))
+                    #errors.warning("compute target '{}' not defined in AML compute targets for AML workspace '{}'".format(compute_target, aml_ws_name))
+                    actual_target = compute_target
 
         # build ENV VARS
         store_creds = self.config.get_storage_creds()
@@ -602,22 +676,42 @@ class AzureML(BackendBase):
         username = args["username"]
         description = args["description"]
         aggregate_dest = args["aggregate_dest"]
+        conda_packages = args["conda_packages"]
+        pip_packages = args["pip_packages"]
         
         env_vars = self.build_env_vars(workspace, aml_ws_name, xt_exper_name, aml_exper_name, run_name, job_id=job_id, 
             compute_target=compute_target, username=username, description=description, aggregate_dest=aggregate_dest, 
             node_id=node_id, args=args)
 
+        container_registry, docker_image = self.get_docker_container_registry(args)
+
         framework = args["framework"]
-        framework = framework.lower()
+        if framework:
+            framework = framework.lower()
+        else:
+            framework = "none"
+        framework_version = args["fw_version"]
+
+        use_docker = bool(docker_image)
+        user_managed = bool(docker_image)
+
+        # AML issues warnings if this is not set
+        aml_use_docker = True
+
+        if docker_image:
+            # when user specified docker_image, he takes responsibility for python/conda and framework (torch, tensorflow, etc.)
+            framework = "none"
+            framework_version = None
 
         is_distributed = args['distributed']
         dist_training = args["distributed_training"]
         dist_training = dist_training.lower()
 
+        import azureml
         from azureml.train.estimator import Estimator, Mpi, Gloo, Nccl
         from azureml.train.dnn import PyTorch, Chainer, TensorFlow
-
-        fw_dict = {"pytorch": PyTorch, "tensorflow": TensorFlow, "chainer": Chainer, "estimator": Estimator}
+        
+        fw_dict = {"pytorch": PyTorch, "tensorflow": TensorFlow, "chainer": Chainer, "none": Estimator}
         dt_dict = {"mpi": Mpi, "gloo": Gloo, "nccl": Nccl}
 
         if not framework in fw_dict:
@@ -652,56 +746,122 @@ class AzureML(BackendBase):
             node_count = 1
 
         vm_size = args["vm_size"]
-        conda_packages = args["conda_packages"]
-        pip_packages = args["pip_packages"]
         use_gpu = args["use_gpu"]
-        framework_version = args["fw_version"]
         max_secs = args["max_seconds"]
-        user_managed = args["user_managed"]
+        # user_managed = args["user_managed"]
 
-        activate_cmd = self.get_activate_cmd()
+        # if pip_packages includes a requirements file, extract it
+        fn_requirements = None
+        for pp in pip_packages:
+            if pp.startswith("-r "):
+                fn_requirements = pp[3:].strip()
+                pip_packages.remove(pp)
+                break
+
+        activate_cmd = self.get_activate_cmd(args)
         if activate_cmd:
             # we have no way of running this on AML before conda_packages and pip_packages are installed (or used to build a docker image)
             errors.config_error("setup.activate property cannot be specified for AML targets")
 
         #max_secs = 10080 if max_secs <= 0 else max_secs
         
-        use_docker = False      
-        environment_name = utils.safe_value(compute_def, "docker")
-        if environment_name:
-            envrionment_def = self.config.get_docker_def(environment_name)
-            if envrionment_def:
-                use_docker = (envrionment_def["type"] == "docker")
+        setup = self.config.get_setup_from_target_def(self.compute_def)
+        use_sudo = utils.safe_value(setup, "use-sudo")       # some AML clusters need sudo
+        allow_other = use_sudo      # if setup with sudo, then we need allow_other
 
-        # workaround AML warning
-        if not use_docker:
-            use_docker = None
+        # must use environment to specify docker registry username/password
+        #use_envionement = bool(container_registry)  #   use_docker
+        use_environment = True   # for AML/docker flags to be used 
+        estimator_keyword_args = {}
 
-        if self.submit_logs:
+        # dynamic args for estimator ctr
+        if framework != "none":
+            estimator_keyword_args["framework_version"] = framework_version
+
+        if fake_submit or self.submit_logs:
             # for testing (this should match exact args used in estimator ctr below)
-            self.serializable_estimator = {"source_directory": code_dir, "script_params": arg_dict, "compute_target": actual_target, 
-                "vm_size": vm_size, "entry_script": target_fn, "conda_packages": conda_packages, "pip_packages": pip_packages, 
-                "use_gpu": use_gpu, "use_docker": use_docker, "framework_version": framework_version, "user_managed": user_managed, 
-                "environment_variables": env_vars, "node_count": node_count, "distributed_training": {},
-                "max_run_duration_seconds": max_secs}
+            if use_environment:
+                serializable_environment = {"environment_variables": env_vars, "docker": {"enabled": aml_use_docker, "gpu_support": use_gpu,
+                    "base_image": docker_image}}
+
+                self.serializable_estimator = {"source_directory": code_dir, "script_params": arg_dict, "compute_target": actual_target, 
+                    "entry_script": target_fn, "node_count": node_count, "distributed_training": {},
+                    "max_run_duration_seconds": max_secs, "user_managed": user_managed, "vm_size": vm_size, 
+                    "environment_definition": serializable_environment, **estimator_keyword_args}
+            else:
+                self.serializable_estimator = {"source_directory": code_dir, "script_params": arg_dict, "compute_target": actual_target, 
+                    "vm_size": vm_size, "entry_script": target_fn, "conda_packages": conda_packages, "pip_packages": pip_packages, 
+                    "pip_requirements_file": fn_requirements, "use_gpu": use_gpu, "use_docker": aml_use_docker, 
+                    "user_managed": user_managed, "environment_variables": env_vars, "node_count": node_count, "distributed_training": {},
+                    "max_run_duration_seconds": max_secs, **estimator_keyword_args}
 
         if fake_submit:
             estimator = self.serializable_estimator
         else:
-            estimator = estimator_ctr(source_directory=code_dir, script_params=arg_dict, compute_target=actual_target, 
-                vm_size=vm_size, entry_script=target_fn, conda_packages=conda_packages, pip_packages=pip_packages, 
-                use_gpu=use_gpu, use_docker=use_docker, framework_version=framework_version, user_managed=user_managed, 
-                environment_variables=env_vars, node_count=node_count, distributed_training=distributed_obj, 
-                max_run_duration_seconds=max_secs)
+            if use_environment:
+
+                environment = azureml.core.Environment("myenv")
+                environment.environment_variables = env_vars
+                environment.python.user_managed_dependencies = user_managed
+
+                environment.docker.enabled = aml_use_docker
+                environment.docker.gpu_support = use_gpu            # deprecated?
+
+                if container_registry:
+                    environment.docker.base_image_registry = container_registry
+
+                if docker_image:
+                    # override AML default base image with user-specified image
+                    environment.docker.base_image = docker_image 
+                else:
+                     environment.docker.base_image = self.get_default_docker_image(use_gpu)
+
+                # specify conda_packages and pip_packages here
+                conda = azureml.core.environment.CondaDependencies.create(pip_packages=pip_packages, 
+                    conda_packages=conda_packages)
+
+                environment.python.conda_dependencies = conda
+
+                if use_sudo:
+                    # specify additional docker run flags so that we can use blobfuse
+                    #docker_flags == "--privileged"
+                    environment.docker.arguments = ["--privileged"]
+                    #environment.docker.arguments = ["--cap-add", "SYS_ADMIN", "--device", "/dev/fuse"]
+                    #environment.docker.arguments = ["--cap-add", "SYS_ADMIN", "--device", "/dev/fuse", "--security-opt", "apparmor:unconfined"]
+
+                estimator = estimator_ctr(source_directory=code_dir, script_params=arg_dict, compute_target=actual_target, 
+                    vm_size=vm_size, distributed_training=distributed_obj, 
+                    entry_script=target_fn, node_count=node_count, max_run_duration_seconds=max_secs, environment_definition=environment)
+                    
+            else:
+                pip_packages = None
+
+                estimator = estimator_ctr( source_directory=code_dir, script_params=arg_dict, compute_target=actual_target, 
+                    vm_size=vm_size, entry_script=target_fn, conda_packages=conda_packages, pip_packages=pip_packages, 
+                    pip_requirements_file=fn_requirements, use_gpu=use_gpu, use_docker=aml_use_docker, 
+                    user_managed=user_managed, environment_variables=env_vars, node_count=node_count, distributed_training=distributed_obj,
+                    max_run_duration_seconds=max_secs, custom_docker_image=docker_image)
+
+            # update estimator for ITP 
+            sku = utils.safe_value(self.compute_def, "sku", "g1")
+            gpu_count = int(sku[1:]) if isinstance(sku, str) else 0
+
+            low_pri = utils.safe_value(self.compute_def, "low_pri", False)
+
+            self.update_estimator(estimator, gpu_count, low_pri)
 
         return estimator, experiment
 
+    def update_estimator(self, estimator, gpu_count, preemption_allowed):
+        pass
+
     def submit_job(self, job_id, job_runs, workspace, compute_def, resume_name, 
-            repeat_count, using_hp, runs_by_box, experiment, snapshot_dir, controller_scripts, args):
+        repeat_count, using_hp, runs_by_box, experiment, snapshot_dir, controller_scripts, args):
 
         username = args["username"]
         aml_exper_name = "{}__{}__{}".format(username, workspace, experiment)
         cwd = os.getcwd()
+        bootstrap_dir = args["bootstrap_dir"]
 
         compute = args["target"]
         compute_def = args["compute_def"]
@@ -716,7 +876,7 @@ class AzureML(BackendBase):
 
         for i, node_runs in enumerate(job_runs):
             node_info = self.submit_node_runs(job_id, node_runs, workspace, aml_ws_name, experiment, aml_exper_name, compute_def, resume_name, 
-                repeat_count, using_hp, compute, runs_by_box, snapshot_dir, i, show_aml_run_name, nodes, args)
+                repeat_count, using_hp, compute, runs_by_box, bootstrap_dir, i, show_aml_run_name, nodes, args)
 
             node_id = "node" + str(i)
             service_info_by_node[node_id] = node_info
@@ -854,7 +1014,8 @@ class AzureML(BackendBase):
             node_info["run_name"] = run_name
 
             # update mongo db info for run with cluster and service_job_id
-            mongo.update_mongo_run_from_dict(workspace, run_name, {"aml_exper_name": aml_exper_name, "aml_run_number": aml_run_number})
+            # DO WE STILL NEED THIS?  Aug-08-2020
+            #mongo.update_mongo_run_from_dict(workspace, run_name, {"aml_exper_name": aml_exper_name, "aml_run_number": aml_run_number})
 
         if monitor_cmd:
             console.print("monitoring notebook created; to run:")
@@ -932,13 +1093,16 @@ class AzureML(BackendBase):
 
         dataref = datastore.as_mount()
         dir_name = dataref.path_on_compute
-        console.print("daatastore MOUNT dir_name=", dir_name)
+        console.print("datastore MOUNT dir_name=", dir_name)
         return dir_name
 
     # API call
     def read_log_file(self, service_node_info, log_name, start_offset=0, end_offset=None, 
         encoding='utf-8', use_best_log=True):
 
+        FN_STDOUT_LOG = "azureml-logs/00_stdout.txt"
+        FN_STDOUT2_LOG = "azureml-logs/70_driver_log.txt"
+
         run = self.get_node_run(service_node_info)
 
         # aml bug workaround
@@ -960,10 +1124,14 @@ class AzureML(BackendBase):
         if use_best_log:
             # list of currently available logs
             available_logs = run._get_logs(current_details)
-
-            # get highest priority (most relevant to user) log name
-            next_log = Run._get_last_log_primary_instance(available_logs) if available_logs else None
-
+            if FN_STDOUT_LOG in available_logs:
+                next_log = FN_STDOUT_LOG
+            elif FN_STDOUT2_LOG in available_logs:
+                next_log = FN_STDOUT2_LOG
+            else:
+                # let AML pick best log 
+                next_log = Run._get_last_log_primary_instance(available_logs) if available_logs else None
+    
             if available_logs and log_name != next_log:
                 # switching logs, so reset offset and remember new name
                 start_offset = 0
@@ -1002,8 +1170,8 @@ class AzureML(BackendBase):
         # translates an AML status to a simple status (queued, running, completed)
 
         queued = ["NotStarted", "Starting", "Provisioning", "Preparing", "Queued"]
-        running = ["Running"]
-        completed = ["Finalizing", "CancelRequested", "Completed", "Failed", "Canceled", "NotResponding"]
+        running = ["Running", "Finalizing"]
+        completed = ["CancelRequested", "Completed", "Failed", "Canceled", "NotResponding"]
 
         if status in queued:
             ss = "queued"
@@ -1064,3 +1232,11 @@ class AzureML(BackendBase):
             result = {"cancelled": cancelled, "service_status": service_status, "simple_status": simple_status}
 
         return result
+
+   # API call
+    def add_service_log_copy_cmds(self, cmds, dest_dir, store_path, args):
+
+        self.append(cmds, "mkdir -p {}".format(dest_dir))
+        #self.append(cmds, "cp -r -v $AZUREML_LOGDIRECTORY_PATH/* {}".format(dest_dir))
+        #self.append(cmds, "cp -r -v $AZ_BATCHAI_STDOUTERR_DIR/* {}".format(dest_dir))
+        self.append(cmds, "cp -r azureml_compute_logs/* {}".format(dest_dir))
