@@ -11,8 +11,8 @@ class ITP(AzureML):
     def __init__(self, compute, compute_def, core, config, username=None, arg_dict=None, disable_warnings=True):
         super(ITP, self).__init__(compute, compute_def, core, config, username, arg_dict, disable_warnings)
 
-        # experiment: hide and see if blobfuse is now working
-        #self.mounting_enabled = False
+        # still busted, as of 9/2/2020
+        self.mounting_enabled = False
 
     def get_name(self):
         return "itp"
@@ -70,5 +70,11 @@ class ITP(AzureML):
         # and adds a new line (#!path-to-python) to the xt file 
         # since ITP installs in incorrectly thru conda
         self.append(cmds, "xt_file=$(which xt)")
+
+        # show the problem
+        self.append(cmds, "echo xt script: $xt_file")
+        self.append(cmds, "cat $xt_file")
+
+        # fix the problem
         self.append(cmds, "{ echo '#!'$(which python); grep -v azureml-envs  $xt_file; } > tmp")
         self.append(cmds, "sudo mv tmp $xt_file")
