@@ -25,10 +25,15 @@ from xtlib import scriptor
 from xtlib import constants
 from xtlib import run_helper
 from xtlib import file_utils
+from xtlib import store_utils
 from xtlib import process_utils
 from xtlib import box_information
 
-#FN_CURLOUTPUT = os.path.realpath(os.path.expanduser("~/.xt/cwd/curl_output.txt"))
+# on philly, these need to be under /mnt
+JOBS_MOUNT_PATH = "/mnt/.xt/mnt/jobs_container"
+WORKSPACE_MOUNT_PATH = "/mnt/.xt/mnt/workspace_container"
+DATA_MOUNT_PATH = "/mnt/.xt/mnt/data_container"
+MODELS_MOUNT_PATH = "/mnt/.xt/mnt/models_container"
 
 class Philly(BackendBase):
     def __init__(self, compute, compute_def, core, config, username=None, arg_dict=None):
@@ -41,8 +46,10 @@ class Philly(BackendBase):
         self.username = username
         self.store = self.core.store
 
-        self.cluster = utils.safe_value(arg_dict, "cluster")
-        self.vc = utils.safe_value(arg_dict, "vc")
+        # caller should have updated compute_def with any cmd-specific options for cluster, vc, and queue
+        self.cluster = utils.safe_value(compute_def, "cluster")
+        self.vc = utils.safe_value(compute_def, "vc")
+        self.queue = utils.safe_value(compute_def, "queue")
         
     def get_name(self):
         return "philly"
@@ -52,7 +59,7 @@ class Philly(BackendBase):
         This method is called to allow the backend to inject needed shell commands before the user cmd.  This 
         base implementation does so by generating a new script file and adding it to the snapshot_dir.
         '''
-        pass        
+        pass   
 
     def get_client_cs(self, service_node_info):
         '''
@@ -64,8 +71,9 @@ class Philly(BackendBase):
         cs = None
 
         job_id = service_node_info["job_id"]
+        ws_name = service_node_info["ws_name"]
         filter = {"_id": job_id}
-        jobs = self.core.store.mongo.get_info_for_jobs(filter, None)
+        jobs = self.core.store.mongo.get_info_for_jobs(ws_name, filter, None)
         if not jobs:
             errors.store_error("unknown job_id: {}", job_id)
 
@@ -115,11 +123,6 @@ class Philly(BackendBase):
             text, rows = lb.build_formatted_table(jobs, columns)
             console.print(text)
 
-        # for job in jobs:
-        #     #console.print("    {:.100s}".format(str(job)))
-        #     utils.print_dict_lines(job, "    ")
-        #     console.print("-----------------")
-
     def view_status(self, run_name, workspace, job, monitor, escape_secs, auto_start, 
             stage_flags, status, max_finished):
 
@@ -163,48 +166,50 @@ class Philly(BackendBase):
         return env
 
     def process_data_action(self, cmds, store_data_dir, data_action):
+        self.append_title(cmds, "MOUNT or DOWNLOAD DATA:")
+
         if data_action == "mount":
             # point to parent data directory (not target, e.g., mnist)
-            data_mnt = "/mnt/data/" + store_data_dir
+            data_mnt = DATA_MOUNT_PATH + "/" + store_data_dir
 
-            self.append(cmds, "echo MOUNTING data SHARE={} to local path={}".format(store_data_dir, data_mnt))
-            self.append(cmds, 'export XT_DATA_MNT="{}"'.format(data_mnt))
+            self.append(cmds, "echo MOUNTING data SHARE={} to local path={}".format(store_data_dir, data_mnt), echo=False)
             self.append(cmds, 'export XT_DATA_DIR="{}"'.format(data_mnt))
-            self.append(cmds, "ls {}".format(data_mnt), echo_before=True)
+            self.append(cmds, "ls {}".format(data_mnt))
         elif data_action == "download":
             dest_dir = "/home/$USER/data/" + store_data_dir
 
-            self.append(cmds, "echo DOWNLOADING data SHARE={} to local path={}".format(store_data_dir, dest_dir))
+            self.append(cmds, "echo DOWNLOADING data SHARE={} to local path={}".format(store_data_dir, dest_dir), echo=False)
             self.append(cmds, 'export XT_DATA_DIR="' + dest_dir + '"')
             self.append(cmds, 'mkdir $XT_DATA_DIR -p')
 
             # make it look like this is parent dir
             dest_dir_ext = dest_dir   # + "/" + store_data_dir
 
-            requests = [ {"container": utils.DATA_STORE_ROOT, "blob_path": store_data_dir, "dest_dir": dest_dir_ext} ]
+            requests = [ {"container": store_utils.DATA_STORE_ROOT, "blob_path": store_data_dir, "dest_dir": dest_dir_ext} ]
             sub_cmds = self.create_download_commands("xt", False, True, requests)
             cmds += sub_cmds
 
     def process_model_action(self, cmds, store_model_dir, model_action):
+        self.append_title(cmds, "MOUNT or DOWNLOAD MODELS:")
+
         if model_action == "mount":
             # point to parent data directory (not target, e.g., mnist)
-            model_mnt = "/mnt/models/" + store_model_dir
+            model_mnt = MODELS_MOUNT_PATH + "/" + store_model_dir
 
-            self.append(cmds, "echo MOUNTING model SHARE={} to local path={}".format(store_model_dir, model_mnt))
-            self.append(cmds, 'export XT_MODEL_MNT="{}"'.format(model_mnt))
+            self.append(cmds, "echo MOUNTING model SHARE={} to local path={}".format(store_model_dir, model_mnt), echo=False)
             self.append(cmds, 'export XT_MODEL_DIR="{}"'.format(model_mnt))
-            self.append(cmds, "ls {}".format(model_mnt), echo_before=True)
+            self.append(cmds, "ls {}".format(model_mnt))
         elif model_action == "download":
             dest_dir = "/home/$USER/models/" + store_model_dir
 
-            self.append(cmds, "echo DOWNLOADING model SHARE={} to local path={}".format(store_model_dir, dest_dir))
+            self.append(cmds, "echo DOWNLOADING model SHARE={} to local path={}".format(store_model_dir, dest_dir), echo=False)
             self.append(cmds, 'export XT_MODEL_DIR="' + dest_dir + '"')
             self.append(cmds, 'mkdir $XT_MODEL_DIR -p')
 
             # make it look like this is parent dir
             dest_dir_ext = dest_dir    # + "/" + store_model_dir
 
-            requests = [ {"container": utils.MODELS_STORE_ROOT, "blob_path": store_model_dir, "dest_dir": dest_dir_ext} ]
+            requests = [ {"container": store_utils.MODELS_STORE_ROOT, "blob_path": store_model_dir, "dest_dir": dest_dir_ext} ]
             sub_cmds = self.create_download_commands("xt", False, True, requests)
             cmds += sub_cmds
 
@@ -212,80 +217,71 @@ class Philly(BackendBase):
         store_data_dir, data_action, store_model_dir, model_action, use_controller, args):
 
         self.is_windows = False
+        self.capture_setup_cmds = args["capture_setup_cmds"]
         cmds = []
-        pip_freeze = args["pip_freeze"]
-
-        # # NOTE: usage of $ENVVAR must be surrounded by DOUBLE quotes
-        # self.append(cmds, 'echo ----- START of XT-level processing -----')
-        # self.append(cmds, 'echo "cwd: $PWD" ')
 
         # DEBUG GPU ISSUE
         # NOTE: installing torchvision==0.4.1 *does* causes pytorch to be REINSTALLED without correct GPU support (on Philly)
-        # verifyed Feb-18-2020, rfernand2.
-        #self.append(cmds, '''python -c "import torch; print('TORCH version', torch.__version__, ' says CUDA available=', torch.cuda.is_available())"''')
-        #self.append(cmds, 'pip install torchvision==0.4.1')
-        #self.append(cmds, '''python -c "import torch; print('TORCH version', torch.__version__, ' says CUDA available=', torch.cuda.is_available())"''')
 
-        self.add_first_cmds(cmds, "long philly cmd string", change_dir=True)
+        self.add_first_cmds(cmds, homebase="$HOME", cwd="$HOME/.xt/cwd", args=args)
 
-        # supress annoying warnings in philly log
-        self.append(cmds, 'export PATH=$PATH:~/.local/bin')
-
-        self.add_setup_cmds(cmds, pip_freeze=pip_freeze, args=args)
-        self.add_xt_setup_cmds(cmds, self.is_windows)
+        self.append_title(cmds, "UNZIP code files")
 
         # create environment variables for user
-        self.append(cmds, 'export XT_CODE_MNT="/mnt/default/' + store_before_dir + '/code"')
-        self.append(cmds, 'export XT_OUTPUT_MNT="/mnt/output/' + store_output_dir + '"')
-        self.append(cmds, 'export XT_AFTER_MNT="/mnt/output/' + store_after_dir + '"')
+        job_id = args["job_id"]
+        node_store_path = "jobs/{}/node-{}".format(job_id, self.node_index)
+        code_store_path = "jobs/{}/before/code".format(job_id, self.node_index)
 
-        # ensure required directories exist
-        self.append(cmds, 'mkdir -p "/home/$USER/.xt"')
-
-        # bug workaround: this creates a 0-len blob in Azure storage that causes problems downstream
-        #self.append(cmds, 'mkdir -p "$XT_OUTPUT_MNT"')
-
-        # COPY CODE files from blob fusion INPUT
-        self.append(cmds, "echo copying source code from: $XT_CODE_MNT")
-        self.append(cmds, 'cp -r "$XT_CODE_MNT" ~/code')
-        self.append(cmds, "cd ~/code")
-
-        code_zip = args["code_zip"]
-        if code_zip and code_zip != "none":
-            # UNZIP code
-            self.append(cmds, "echo unzipping " + constants.CODE_ZIP_FN)
-            self.append(cmds, 'sudo apt install unzip')
-            self.append(cmds, 'unzip {} > __unzip__.log'.format(constants.CODE_ZIP_FN))
+        self.append(cmds, 'export XT_OUTPUT_DIR={}/{}'.format(WORKSPACE_MOUNT_PATH, store_output_dir))
+        self.append(cmds, 'export XT_NODE_DIR={}/{}'.format(JOBS_MOUNT_PATH, node_store_path))
+        self.append(cmds, 'export XT_CODE_DIR={}/{}'.format(JOBS_MOUNT_PATH, code_store_path))
 
         # prevent python from writing byte code (faster for 1-off usage?)        
         self.append(cmds, "export PYTHONDONTWRITEBYTECODE=1")
-        self.append(cmds, 'export PYTHONPATH="$XT_CODE_MNT:$PYTHONPATH"')
 
+        hold = args["hold"]
+        if hold:
+            # special command needed for BASH JOBS to tell philly to hold node open after run ends
+            self.append(cmds, "echo --debug >/dev/null")
+
+        # COPY CODE files from mapped INPUT path
+        self.append(cmds, 'cp -r $XT_CODE_DIR/. .')
+
+        self.append_unzip(cmds, constants.CODE_ZIP_FN, ".")
+        self.append(cmds, "rm {}".format(constants.CODE_ZIP_FN))
+        
+        # now that code files are present and unzipped, we can run the setup
+        # commands (which may rely on files like requirements.txt)
+        super().add_setup_cmds(cmds, remove_zip=False, args=args)
+
+        super().add_other_cmds(cmds, args)
+        super().add_report_cmds(cmds, True, args)
+
+        # NOTE: these action commands rely on XT being installed 
         self.process_data_action(cmds, store_data_dir, data_action)
         self.process_model_action(cmds, store_model_dir, model_action)
 
-        if pip_freeze:
-            self.append(cmds, "echo final PIP FREEZE:")
-            self.append(cmds, "pip freeze > __final_pip_freeze__.log")
+        wrapped_cmd = self.create_wrapped_cmd(cmds, user_cmd, args)
+        return wrapped_cmd
 
-        self.add_report_cmds(cmds, True)
+    def create_wrapped_cmd(self, cmds, user_cmd, args):
 
-        # run user's cmd
-        if not use_controller:   
-            # --direct option; add a shell prefix in front of script
-            if not user_cmd.endswith(".py"):
-                # do we need this?  it assumes user_cmd contains a 'conda activate cmd'
-                prefix = self.config.get("script-launch-prefix", "philly")
-                user_cmd = process_utils.make_launch_cmd(prefix, user_cmd, is_script=True)
-            
+        self.append_title(cmds, "LAUNCHING XT CONTROLLER:")
+
+        self.append_dir(cmds)
         self.append(cmds, user_cmd)
+        self.add_log_upload_cmds(cmds, args)
 
-        self.append(cmds, 'echo ----- END of XT-level processing -----')
+        self.append_title(cmds, 'END of XT-level processing', double=True)
+        self.append_title(cmds, 'START of XT-level processing', double=True, index=0)
+
+        # remove empty cmds
+        cmds = [cmd for cmd in cmds if cmd]
 
         wrapped_cmd = "; ".join(cmds)
         return wrapped_cmd
 
-    def build_azure_volumes(self, input_container, data_container, output_container, data_action, data_writable,
+    def build_azure_volumes(self, ws_container, data_container, data_action, data_writable,
             model_container, model_action, model_writable, args):
         # specify how to mount our azure storage for blobfuse system
         storage_creds = self.config.get_storage_creds()
@@ -299,23 +295,29 @@ class Philly(BackendBase):
 
         options_readonly = options + ["-o", "ro"]
 
-        # mount CODE and AFTER containers
-        volumes = {
-            # these are the default set of philly tools mounts 
-            "default":  {"type": "blobfuseVolume", "storageAccount": store_name, "containerName": input_container, "path": "/mnt/default", "options": options_readonly}, 
-            "output": {"type": "blobfuseVolume", "storageAccount": store_name, "containerName": output_container, "path": "/mnt/output", "options": options}, 
-            "_output": {"type": "blobfuseVolume", "storageAccount": store_name, "containerName": output_container, "path": "/mnt/_output", "options": options}
-            }
+        # mount CODE, OUTPUT, and NODE containers
+        volumes = {}
+
+        # always add workspace
+        volumes["workspace"] = {"type": "blobfuseVolume", "storageAccount": store_name, "containerName": ws_container, 
+            "path": WORKSPACE_MOUNT_PATH, "options": options}
+
+        # always add jobs
+        jobs_container = store_utils.get_jobs_container(ws_name=ws_container)
+        volumes["jobs"] = {"type": "blobfuseVolume", "storageAccount": store_name, "containerName": jobs_container, 
+            "path": JOBS_MOUNT_PATH, "options": options}
 
         # mount DATA share
         if data_action == "mount":
             opts = options if data_writable else options_readonly
-            volumes["data"] = {"type": "blobfuseVolume", "storageAccount": store_name, "containerName": data_container, "path": "/mnt/data", "options": opts}
+            volumes["data"] = {"type": "blobfuseVolume", "storageAccount": store_name, "containerName": data_container, 
+                "path": DATA_MOUNT_PATH, "options": opts}
 
         # mount MODELS share
         if model_action == "mount":
             opts = options if model_writable else options_readonly
-            volumes["model"] = {"type": "blobfuseVolume", "storageAccount": store_name, "containerName": model_container, "path": "/mnt/models", "options": opts}
+            volumes["model"] = {"type": "blobfuseVolume", "storageAccount": store_name, "containerName": model_container, 
+                "path": MODELS_MOUNT_PATH, "options": opts}
 
         credentials = {"storageAccounts": {store_name: {"key": store_key}}}
 
@@ -414,13 +416,10 @@ class Philly(BackendBase):
         model_action = args["model_action"]
         model_writable = args["model_writable"]
         hold = args["hold"]
+        target = args["target"]
 
         use_controller = (not args["direct_run"])
 
-        # currently, we put all 'data stores' under __data__
-        # if store_data_dir:
-        #     store_data_dir = utils.DATA_STORE_ROOT + "/" + store_data_dir
-
         wrapped_cmds = []
         run_names = []
 
@@ -432,13 +431,20 @@ class Philly(BackendBase):
 
             cmd_line = " ".join(cmd_parts)
             
-            store_output_dir = "runs/" + run_name + "/output"
-            store_after_dir = "runs/" + run_name + "/after"
+            # for wrapper building functions to access 
+            self.node_index = utils.node_index(node_id)
+            self.node_id = node_id
+            self.run_name = run_name
+
+            run_path = store_utils.get_run_path(job_id, self.run_name) 
+            store_output_dir = "{}/output".format(run_path)
+            store_after_dir = "{}/after".format(run_path)
 
             wrapped_cmd = self.wrap_cmd_line_for_philly_job(cmd_line, store_before_dir, 
                 store_output_dir, store_after_dir, store_data_dir=store_data_dir, 
                 data_action=data_action, store_model_dir=store_model_dir, 
-                model_action=model_action, use_controller=use_controller, args=args)
+                model_action=model_action, use_controller=use_controller, 
+                args=args)
     
             wrapped_cmds.append(wrapped_cmd)
             run_names.append(run_name)
@@ -448,39 +454,17 @@ class Philly(BackendBase):
         sku = utils.safe_value(compute_def, "sku").upper()  
         queue = utils.safe_value(compute_def, "queue")      
 
-        env_name = utils.safe_value(self.compute_def, "docker")
-        if not env_name:
-            errors.config_error("Philly compute target '{}' is missing the required 'docker' property (in XT config file)".format(self.compute))
-
-        env = self.config.get("dockers", env_name, default_value=None)
-        if not env:
-            errors.config_error("docker '{}' definition not found in XT config file".format(env_name))
-
-        docker_image = utils.safe_value(env, "image")
-        if not docker_image:
-            errors.config_error("Definition for environment '{}' is missing the required 'image' property".format(env_name))
-
-        registry_name = utils.safe_value(env, "registry")
-        if not registry_name:
-            errors.config_error("Definition for environment '{}' is missing the required 'registry' property".format(env_name))
-
-        docker_registry = self.config.get("external-services", registry_name, default_value=None)
-        if not docker_registry:
-            errors.config_error("Definition for environment '{}' is missing the required 'registry' property".format(env_name))
-
-        login_server = utils.safe_value(docker_registry, "login-server")
-        if not login_server:
-            errors.config_error("Definition for registry '{}' is missing the required 'login-server' property".format(registry_name))
+        docker_name = args["docker"]
+        docker_image, login_server, docker_registry = self.config.get_docker_info(target, docker_name, required=True)
 
         # philly wants these as a single name
-        docker_image = login_server + "/" + docker_image
+        docker_image = login_server + "/" + docker_image if login_server else docker_image
 
-        # image should look like this: phillyregistry.azurecr.io/microsoft_pytorch:v1.2.0_gpu_cuda9.0_py36_release_gpuenv_hvd0.16.2
-        #console.print("philly docker_image=", docker_image)
+        creds_required = utils.make_numeric_if_possible( utils.safe_value(docker_registry, "login") )
 
         # include job_id in name for assist in cancelling by job_id
         # use a pattern to reliably strip off philly prefix and suffix
-        philly_name = "/{}/{}/{}/".format(job_id, workspace, run_name)
+        philly_name = "/{}/{}/{}/".format(workspace, job_id, run_name)
 
         # for now, let's focus on single command per nodes
         first_run_name = run_names[0]
@@ -508,17 +492,22 @@ class Philly(BackendBase):
 
         count = 1    # change this for distributed runs
 
-        #workers = {"type": "skuResource", "sku": sku, "count": nodes, "image": docker_image, "environmentVariables": env_vars, "commandLine": wrapped_cmd}
         workers = {"type": "skuResource", "sku": sku, "count": count, "image": docker_image, "commandLine": first_wrapped_cmd, "interactive": hold}
-
         resources = {"workers": workers}
-        input_container = constants.INFO_CONTAINER
-        output_container = workspace
 
-        volumes, credentials = self.build_azure_volumes(input_container, utils.DATA_STORE_ROOT, output_container, data_action, 
-            data_writable, utils.MODELS_STORE_ROOT, model_action, model_writable, args)
+        volumes, credentials = self.build_azure_volumes(workspace, store_utils.DATA_STORE_ROOT, data_action, 
+            data_writable, store_utils.MODELS_STORE_ROOT, model_action, model_writable, args)
 
         jd = {"version": "2017-10-01", "metadata": md, "environmentVariables": env_vars, "resources": resources, "volumes": volumes, "credentials": credentials }
+
+        if creds_required:
+            # add username/password to log into docker registry service (Azure)
+            username = utils.safe_value(docker_registry, "username")
+            password = utils.safe_value(docker_registry, "password")
+
+            creds = {"dockerRegistries": {login_server: {"username": username, "password": password}}}
+            jd["credentials"] = creds
+
         text = json.dumps(jd, indent=4)
         #console.print("json=", text)
 
@@ -538,7 +527,7 @@ class Philly(BackendBase):
             #curl_cmd = 'curl -k --ntlm --user : -X POST -H "Content-Type: application/json" --data @{} https://philly/api/jobs > {} 2>&1'.format(fn, fn_log)
             curl_cmd = self.build_curl('-X POST -H "Content-Type: application/json" --data @{} https://philly/api/jobs'.format(fn))
 
-            # SUBMIT the job to PHILLY
+            # SUBMIT the job as a PHILLY BASH JOB (using the Philly REST API)
             result = self.run_curl_cmd(curl_cmd)
             if not "jobId" in result:
                 errors.service_error("Unexpected result from Philly job submit: " + str(result))
@@ -548,7 +537,10 @@ class Philly(BackendBase):
 
             for run_name in run_names:
                 # update mongo db info for run with cluster and service_job_id
-                mongo.update_mongo_run_from_dict(workspace, run_name, {"cluster": cluster, "vc": vc, "service_run_id": service_job_id})
+
+                # add newly acquired info (service_run_id)
+                mongo.update_mongo_run_from_dict(workspace, run_name, 
+                    {"cluster": cluster, "vc": vc, "service_run_id": service_job_id})
 
                 #fb.feedback("{}/{}".format(workspace, run_name), id="run_name", add_seperator=False)
                 fb.feedback("{}/{}".format(workspace, run_name))
@@ -736,7 +728,7 @@ class Philly(BackendBase):
 
         return cancel_results_by_box
 
-    def cancel_runs_by_user(self, box_name):
+    def cancel_runs_by_user(self, ws_name, box_name):
         '''
         Args:
             box_name: the name of the box the runs ran on (pool service)
@@ -776,19 +768,25 @@ class Philly(BackendBase):
 
         return ssh_cmd
 
-    # BACKEND API
-    def get_node_status(self, service_node_info):
+    def get_node_info(self, service_node_info, content="partial"):
         # https://philly/api/status?clusterId=gcr&vcId=pnrsy&jobId=application_1447977864059_0276&jobType=cntk&content=partial
 
         cluster = service_node_info["cluster"]
         vc = service_node_info["vc"]
         philly_job_id = service_node_info["service_id"]
 
-        curl_cmd = self.build_curl('"https://philly/api/status?clusterId={}&vcId={}&jobId={}&content=partial"' \
-            .format(cluster, vc, philly_job_id))
+        curl_cmd = self.build_curl('"https://philly/api/status?clusterId={}&vcId={}&jobId={}&content={}"' \
+            .format(cluster, vc, philly_job_id, content))
 
         result = self.run_curl_cmd(curl_cmd, parse_result=True)
-        status = result["status"]
+        return result
+
+    # BACKEND API
+    def get_node_status(self, service_node_info, content="partial"):
+        # https://philly/api/status?clusterId=gcr&vcId=pnrsy&jobId=application_1447977864059_0276&jobType=cntk&content=partial
+
+        result = self.get_node_info(service_node_info)
+        status = utils.safe_value(result, "status", "unknown")
         return status
 
     # BACKEND API
@@ -805,21 +803,37 @@ class Philly(BackendBase):
         info_verb = "log"
         extra = ""
 
-        philly_status = self.get_node_status(service_node_info)
+        try:
+            philly_status = self.get_node_status(service_node_info)
+        except Exception as ex:
+            console.print("Exception in read_log_file: get_node_status ex=" + str(ex))
+            philly_status = "unknown"
+
         simple_status = self.get_simple_status(philly_status)
 
         curl_cmd = self.build_curl('"https://philly/api/{}?clusterId={}&vcId={}&jobId={}&logType={}&logRev=latest&content=full&jobType=ns{}"' \
             .format(info_verb, cluster, vc, philly_job_id, log_type, extra))
         
-        result = self.run_curl_cmd(curl_cmd, parse_result=True)
-        # if result == "\n":      
-        #     # try log
-        #     log_type = "log"
+        try:
+            result = self.run_curl_cmd(curl_cmd, parse_result=True)
+        except BaseException as ex:
+            console.print("Exception in read_log_file: run_curl_cmd ex=" + str(ex))
+            philly_status = "unknown"
+            result = ""
 
-        #     curl_cmd = self.build_curl('"https://philly/api/{}?clusterId={}&vcId={}&jobId={}&logType={}&logRev=latest&content=full&jobType=ns{}"' \
-        #         .format(info_verb, cluster, vc, philly_job_id, log_type, extra))
+        if result == "\n":      
+            # some clusters need log_type "log" (vs. "stdout")
+            log_type = "log"
 
-        #     result = self.run_curl_cmd(curl_cmd, parse_result=True)
+            curl_cmd = self.build_curl('"https://philly/api/{}?clusterId={}&vcId={}&jobId={}&logType={}&logRev=latest&content=full&jobType=ns{}"' \
+                .format(info_verb, cluster, vc, philly_job_id, log_type, extra))
+
+            try:
+                result = self.run_curl_cmd(curl_cmd, parse_result=True)
+            except:
+                console.print("Exception in read_log_file: run_curl_cmd ex=" + str(ex))
+                philly_status = "unknown"
+                result = ""
 
         if len(result) > 1:
             node_status = "running"
@@ -839,7 +853,7 @@ class Philly(BackendBase):
         # translates an Philly status to a simple status (queued, running, completed)
 
         queued = ["Queued"]
-        running = ["Running"]
+        running = ["Running", "unknown"]
         completed = ["Pass", "Failed", "Killed"]
 
         if status in queued:
@@ -885,3 +899,42 @@ class Philly(BackendBase):
 
         result = {"cancelled": cancelled, "service_status": philly_status, "simple_status": simple_status}
         return result
+
+    def download_log_file(self, file_names, service_node_info, info_verb, log_type, dest_dir):
+        
+        cluster = service_node_info["cluster"]
+        vc = service_node_info["vc"]
+        philly_job_id = service_node_info["service_id"]
+
+        extra = ""
+
+        curl_cmd = self.build_curl('"https://philly/api/{}?clusterId={}&vcId={}&jobId={}&logType={}&logRev=latest&content=full&jobType=ns{}"' \
+            .format(info_verb, cluster, vc, philly_job_id, log_type, extra))
+
+        try:
+            result = self.run_curl_cmd(curl_cmd, parse_result=True)
+        except BaseException as ex:
+            console.print("Exception in read_log_file: run_curl_cmd ex=" + str(ex))
+            philly_status = "unknown"
+            result = ""
+
+        fn_result = "{}/stdout.txt".format(dest_dir)
+        file_utils.write_text_file(fn_result, result)
+
+   # API call
+    def add_service_log_copy_cmds(self, cmds, dest_dir, store_path, args):
+        # self.append(cmds, "ls -lt $PHILLY_JOB_DIRECTORY/logs/1")
+        # self.append(cmds, "ls -lt $PHILLY_JOB_DIRECTORY/stdout/1")
+        # self.append(cmds, "ls -lt $PHILLY_JOB_DIRECTORY/amlogs/1")
+
+        # wait for stdout to flush to disk (unreliable with 10 secs)
+        sleep_secs = 30
+        self.append(cmds, "echo wait 30 secs for our STDOUT to flush to file", echo=False)
+        self.append(cmds, "sleep {}".format(sleep_secs))
+
+
+        self.append(cmds, "tail $PHILLY_JOB_DIRECTORY/stdout/1/stdout.txt")
+
+        cmd = "cp -r $PHILLY_JOB_DIRECTORY/stdout/* {}".format(dest_dir)
+        self.append(cmds, cmd)
+
