@@ -12,6 +12,7 @@ from xtlib.helpers.scanner import Scanner
 
 from xtlib import utils
 from xtlib import errors
+from xtlib import run_helper
 
 '''
 Definitions:
@@ -43,7 +44,7 @@ debug_decorators = False
 first_command = True
 
 def is_xt_object(name):
-   if name.startswith("run") and name[3:].replace('.','',1).isdigit():
+   if run_helper.is_run_name(name):
       match = True
 #    elif name.startswith("job") and name[3:].isdigit():
 #       match = True
@@ -101,27 +102,27 @@ def build_commands():
 
     return func_dict
 
-def remove_hidden_commands():
-    # remove hidden commands from commnds_by_name
-    global commands_by_name
-    commands_by_name = {name:cmd for name,cmd in commands_by_name.items() if not cmd["hidden"]}
+# def remove_hidden_commands():
+#     # remove hidden commands from commnds_by_name
+#     global commands_by_name
+#     commands_by_name = {name:cmd for name,cmd in commands_by_name.items() if not cmd["hidden"]}
 
-    # rebuild commands from commands_by_name
-    ddx = {}
-    for _, cmd in commands_by_name.items():
-        cmd_name = cmd["name"]
+#     # rebuild commands from commands_by_name
+#     ddx = {}
+#     for _, cmd in commands_by_name.items():
+#         cmd_name = cmd["name"]
 
-        # add cmd keywords to dd
-        dd = ddx
-        for name_part in cmd_name.split(" "):
-            if name_part not in dd:
-                dd[name_part] = {}
-            dd = dd[name_part]
+#         # add cmd keywords to dd
+#         dd = ddx
+#         for name_part in cmd_name.split(" "):
+#             if name_part not in dd:
+#                 dd[name_part] = {}
+#             dd = dd[name_part]
 
-        dd[""] = cmd
+#         dd[""] = cmd
 
-    global commands
-    commands = ddx
+#     global commands
+#     commands = ddx
 
 def get_commands_from(dd, func_dict):
     for key, value in dd.items():
@@ -281,7 +282,7 @@ def keyword_arg(name, keywords, required=True, type=str, help="", default=None):
     return decorator_keyword_arg
 
 # OPTION decorator processor
-def option(name, default=None, required=None, multiple=False, type=str, values=None, help=""):
+def option(name, arg_name=None, default=None, required=None, multiple=False, type=str, values=None, help=""):
     '''
     params:
         multiple: when True, user can specify this option multiple times and values will accumulate (list of strings)
@@ -300,7 +301,9 @@ def option(name, default=None, required=None, multiple=False, type=str, values=N
             errors.internal_error("@option decorators must be followed by a single @command decorator")
 
         type_name = type if isinstance(type, str) else type.__name__
-        option_info = {"name": name, "hidden": False, "required": required, "type": type_name, "multiple": multiple, 
+        actual_arg_name = arg_name if arg_name else name
+
+        option_info = {"name": name, "arg_name": actual_arg_name, "hidden": False, "required": required, "type": type_name, "multiple": multiple, 
             "default": default, "values": values, "help": help}
         
         #current_cmd_info["options"].append(option_info)
@@ -478,17 +481,44 @@ class Dispatcher():
         commands_by_name = build_commands()
         current_dispatcher = self
 
+    def build_xt_cmd_from_dict(self, cmd, arguments, options, arg_dict):
+        xt_cmd = cmd
+        args = ""
+        opts = ""
+
+        for arg in arguments:
+            arg_name = arg["name"]
+            if arg_name in arg_dict:
+                args += " " + str(arg_dict[arg_name])
+
+        for opt in options:
+            opt_name = opt["name"]
+            if opt_name in arg_dict:
+                opts += " --{}={}".format(opt_name, arg_dict[opt_name])
+        
+        if cmd == "run":
+            xt_cmd += opts + args
+        else:
+            xt_cmd += args + opts
+
+        return xt_cmd
+
     def validate_and_add_defaults_for_cmd(self, cmd, arg_dict):
         cmd_info = get_command("run")
         options = cmd_info["options"]
         arguments = cmd_info["arguments"]
 
+        # build xt_cmd for logging purposes
+        self.dispatch_cmd = self.build_xt_cmd_from_dict(cmd, arguments, options, arg_dict)
+
         return self.validate_and_add_defaults(arguments, options, arg_dict)
 
     def show_current_command_syntax(self):
         console.print()
-        help_impl = self.impl_dict["xtlib.impl_help"]
-        help_impl.command_help(self.cmd_info, True, False)
+
+        if self.cmd_info:
+            help_impl = self.impl_dict["xtlib.impl_help"]
+            help_impl.command_help(self.cmd_info, True, False)
 
     def hide_commands(self, cmds_to_hide):
         for cmd in cmds_to_hide:
@@ -517,9 +547,10 @@ class Dispatcher():
 
     def match(self, text, cmd):
         '''
-        implements the min. 4-char abbreviaton matching.
+        user can abbreviate down to 3 letters, but text must unambiguosly select
+        an keyword.
         '''
-        return cmd == text or (len(text) >= 4 and cmd.startswith(text))
+        return cmd == text or (len(text) >= 3 and cmd.startswith(text))
 
     def match_keyword(self, value, keywords):
         found = None
@@ -729,20 +760,33 @@ class Dispatcher():
         return tag_list, tok
 
     def list_item_by_name(self, name, values):
-        matches = [value for value in values if self.match(name, value["name"])]
+        # first, look for an exact match (takes priority)
+        matches = [value for value in values if name == value["name"]]
+        if not matches:
+            # look for abbreviated matches
+            matches = [value for value in values if self.match(name, value["name"])]
+            if len(matches) > 1:
+                errors.syntax_error("abbreviated name '{}' results in multiple matches: {}".format(name, ", ".join(matches)))
+
         found = matches[0] if matches else None
         return found
 
     def list_item_by_value(self, value, values):
         matches = [val for val in values if self.match(value, val)]
+        if len(matches) > 1:
+            errors.syntax_error("abbreviated value '{}' results in multiple matches: {}".format(value, ", ".join(matches)))
         found = matches[0] if matches else None
         return found
 
     def get_default_from_config(self, value):
         if value and isinstance(value, str) and value.startswith("$"):
-            group, prop = value[1:].split(".")
-            value = self.config.get(group, prop)
-
+            if "." in value:
+                group, prop = value[1:].split(".")
+                value = self.config.get(group, prop)
+            else:
+                group = value[1:]
+                value = self.config.get(group)
+                
         return value
 
     # def process_value(self, value, type):
@@ -1047,18 +1091,31 @@ class Dispatcher():
 
         # process all aguments, options, and flags; ensure each has a value in arg_dict
         all_args = arguments + options
-        all_arg_names = [aa["name"] for aa in all_args]
+        all_arg_dict = {aa["name"]: aa for aa in all_args}
 
-        # process user args in arg_dict
-        for name, value in full_arg_dict.items():
+        # process user args in arg_dict 
+        # caution: this loop may update full_arg_dict so capture keys up front
+        keys = list(full_arg_dict.keys())
+
+        for name in keys:
+            value = full_arg_dict[name]
 
             # validate arg name
-            if not name in all_arg_names:
+            if not name in all_arg_dict:
                 errors.api_error("unknown args name: {}".format(name))
 
+            # does name need to be replaced by arg_name of option/argument?
+            info = all_arg_dict[name]
+            if "arg_name" in info:
+                arg_name = info["arg_name"]
+                if name != arg_name:
+                    # update the name to arg_name
+                    full_arg_dict[arg_name] = value
+                    del full_arg_dict[name]
+
         # now add default values for all other args
         for info in all_args:
-            name = info["name"]
+            name = info["arg_name"] if "arg_name" in info else info["name"]
             required = info["required"] if "required" in info else None
 
             if not name in full_arg_dict:
@@ -1121,8 +1178,6 @@ class Dispatcher():
     def dispatch(self, args, is_rerun=False, capture_output=False, raise_syntax_exception=False):
         self.raise_syntax_exception = raise_syntax_exception
         
-        # TODO: change to cmd_parts parsing, which naturally separates options cleanly (utils.cmd_split)
-
         # be sure to reset this for each parse (for multi-command XT sessions)
         global explict_options
         explict_options = {}
@@ -1321,7 +1376,7 @@ class Dispatcher():
                 tok = scanner.scan()
                 value = self.parse_flag_value(name, value)
             else:
-                value = 1
+                value = True
         else:
             # its a simple value
             value = tok
