@@ -24,8 +24,8 @@ logger = logging.getLogger(__name__)
 
 class RunInfo():
     def __init__(self, run_name, workspace, cmd_parts, run_script, repeat, context, status, show_output=True, parent_name=None, 
-        parent_prep_needed=False, mirror_close_func=None, aml_run=None, node_id=None, run_index=None, mri=None, mri_entry=None, 
-        store=None):
+        parent_prep_needed=False, mirror_close_func=None, aml_run=None, node_id=None, run_index=None, 
+        mri=None, mri_entry=None, store=None):
 
         if isinstance(repeat, str):
             repeat = int(repeat)
@@ -36,6 +36,7 @@ class RunInfo():
         self.cmd_parts = cmd_parts
         self.run_script = run_script
         self.repeat = repeat
+        self.is_parent = bool(repeat)
         self.repeats_remaining = repeat if context.repeats_remaining is None else context.repeats_remaining 
         self.context = context
         self.status = status    
@@ -70,12 +71,6 @@ class RunInfo():
         self.mri_entry = mri_entry
         self.killed_for_restart = False
 
-        console.print("RunInfo: is_aml=", self.is_aml, ", aml_run=", self.aml_run)
-
-        #console.print("mirror_close_func=", mirror_close_func)
-        
-        #console.print("run_info ctr: setting self.repeats_remaining=", self.repeats_remaining)
-        
     def set_console_fn(self, console_fn):
         console_fn = os.path.expanduser(console_fn)
         self.console_fn = console_fn
@@ -118,7 +113,7 @@ class RunInfo():
             for i in range(len(self.acallbacks)):
                 try:
                     callback = self.acallbacks[i]
-                    print("sending text to callback: " + text_msg)
+                    #print("sending text to callback: " + text_msg)
                     callback(self.run_name, text_msg)
                 except BaseException as ex:
                     # if stream is closed unexpectly, treat as non-fatal error and just log
@@ -187,15 +182,19 @@ class RunInfo():
             # in spirit of a box that has just been preempted
 
             #console.print("run_wrapup: self.is_aml=", self.is_aml)
-            is_parent = not (not self.repeat)
+            print("is_parent={}, self.repeat={}".format(self.is_parent, self.repeat))
+
+            #log_info("run_info: calling WRAPUP_RUN")
 
             store.wrapup_run(context.ws, run_name, context.aggregate_dest, context.dest_name, 
                 status=self.status, exit_code=self.exit_code, primary_metric=context.primary_metric, 
                 maximize_metric=context.maximize_metric, report_rollup=context.report_rollup, rundir=self.rundir, 
                 after_files_list=context.after_files_list, after_omit_list=context.after_omit_list, log_events=context.log, 
-                capture_files=context.after_upload, job_id=self.job_id, is_parent = is_parent, node_id=self.node_id, 
+                capture_files=context.after_upload, job_id=self.job_id, is_parent=self.is_parent, node_id=self.node_id, 
                 run_index=self.run_index)
 
+            #if self.mri_entry and store.mongo:
+            #   store.mongo.mark_child_run_completed(self.mri_entry)
             if self.mri and self.mri_entry:
                 self.mri.mark_child_run_completed(self.mri_entry)
 
@@ -241,7 +240,7 @@ class RunInfo():
         self.process = process
         self.status = "running"
         self.process_was_created = True
-        console.print("set_process for {}, pid={}".format(self.run_name, self.process.pid))
+        #console.print("set_process for {}, pid={}".format(self.run_name, self.process.pid))
 
     def kill(self):
 
